#include <stdio.h>
#include <pthread.h>
#include <semaphore.h>

#include "asim/provides/connected_application.h"
#include "asim/provides/stats_device.h"
//#include "asim/provides/SndfileWavUtil.h"

#include "asim/rrr/client_stub_AUDIOCORERRR.h"

using namespace std;

pthread_mutex_t CONNECTED_APPLICATION_CLASS::lock;
pthread_cond_t  CONNECTED_APPLICATION_CLASS::cond;
sem_t CONNECTED_APPLICATION_CLASS::throttle;

// constructor
CONNECTED_APPLICATION_CLASS::CONNECTED_APPLICATION_CLASS(VIRTUAL_PLATFORM vp) :
    clientStub(new AUDIOCORERRR_CLIENT_STUB_CLASS(this))
{
}

// destructor
CONNECTED_APPLICATION_CLASS::~CONNECTED_APPLICATION_CLASS()
{
}

// init
void
CONNECTED_APPLICATION_CLASS::Init()
{

  pthread_mutex_init(&lock, NULL);
  pthread_cond_init(&cond, NULL);
  sem_init(&throttle, 0, 64);

  // enable stats 
  STATS_DEVICE_SERVER_CLASS::GetInstance()->SetupStats();
}

void
CONNECTED_APPLICATION_CLASS::UpdateSemaphore()
{
  sem_post(&throttle);
}

void
CONNECTED_APPLICATION_CLASS::EndSimulation()
{
  printf("EndSimulation Called\n");
  fflush(stdout);
  pthread_mutex_lock(&lock);
  // Do something about the race occuring here
  pthread_cond_signal(&cond); 
  pthread_mutex_unlock(&lock);
  printf("EndSimulation done\n");
  fflush(stdout);  
}

// main
void
CONNECTED_APPLICATION_CLASS::Main()
{
  FILE *inputFile;
  UINT16 sample;
  
  // Convert input wav to pcm
  generate_pcm("input.wav","input.pcm");

  //Send data to the machine here.
  inputFile = fopen("input.pcm","r");
  assert(inputFile);


  int count = 0;

  printf("main: about to enter loop %d\n", count);  

  while(fread(&sample, 2, 1, inputFile)) {
    if(count%1000 == 0)
      printf("main: %d\n", count);
    count++;
    sem_wait(&throttle);
    clientStub->SendUnprocessedStream(Data,(UINT32)sample);
  } 

  printf("main: out of loop\n");

  // Need to put lock here to prevent potential race condition
  pthread_mutex_lock(&lock);
  clientStub->SendUnprocessedStream(EndOfFile,0);

  printf("main: wait for end of file\n");

  pthread_cond_wait(&cond, &lock);
  pthread_mutex_unlock(&lock);

  printf("main: lastt data out\n");

  // Convert input wav to pcm
  generate_wav("out_hw.pcm","input.wav","out_hw.wav");

  printf("generate wav done\n");

  fflush(stdout);
  exit(0);
}

/*    THIS IS THE CODE HANDLING FROM THE REGULAR SOFT-CORE
TO BE INCORPORATED 
// main
void
CONNECTED_APPLICATION_CLASS::Main()
{
  int sleepCount = 0;
  int result = 0;

  fflush(stdout); 

  while ((result = clientStub->ReadCPUToHost(0)) != 1) {
    sleep(1);
    //printf("System controller sleeps with result: %d\n", result);
    sleepCount++;
    if(sleepCount == 100) {
      printf("Failed to get response from hardware, bailing\n\n");
      printf("This means that either your hardware is hanging\n");
      printf("or that the software hasn't given it enough time\n");
      printf("to complete.  If you think it needs more time, then\n");
      printf("edit CONNECTED_APPLICATION_CLASS::Main() in ProcessorSystem.cpp\n");
      printf("(connected_application)\n");
      exit(0);
    }
  }
 
  if(result == 1) {
    printf("\n***PASSED***\n");
  }

  // Dump the stats file

  STATS_DEVICE_SERVER_CLASS::GetInstance()->DumpStats();
  STATS_DEVICE_SERVER_CLASS::GetInstance()->EmitFile();

  fflush(stdout); 
  exit(0);
}*/
